# pylint: disable=missing-module-docstring, missing-function-docstring, line-too-long, too-few-public-methods, invalid-name, pointless-string-statement
import ast

from src.flake8_ast_error import PREFIX, Flake8ASTErrorInfo


class UsingFilterOrMapNotAllowed:
    """
    # Bad
    list(filter(lambda x: x > 5, range(10)))
    list(map(lambda x: x**2, range(10)))

    # Good
    [x for x in range(10) if x > 5]
    [x**2 for x in range(10)]
    """

    msg = PREFIX + "12: Don't use list(filter()) or list(map()), use list comprehension instead"

    @classmethod
    def check(cls, node: ast.Call, errors: list[Flake8ASTErrorInfo]) -> None:
        if (
            isinstance(node.func, ast.Name)
            and node.func.id == "list"
            and node.args
            and isinstance(node.args[0], ast.Call)
            and isinstance(node.args[0].func, ast.Name)
            and node.args[0].func.id in ["filter", "map"]
        ):
            errors.append(Flake8ASTErrorInfo(node.lineno, node.col_offset, cls.msg, type(cls)))



# pylint: disable=missing-module-docstring, missing-function-docstring
import ast
from typing import NamedTuple

PREFIX = "PBP1"

IS = type(ast.Is())
EQUAL = type(ast.Eq())
NOT_EQUAL = type(ast.NotEq())


def filter_by(node: ast.AST, cls: type) -> list[ast.AST]:
    options = [n for n in node if isinstance(n, cls)]
    if options:
        return options[0]
    return []


class Flake8ASTErrorInfo(NamedTuple):
    """
    A named tuple to hold information about a Flake8 AST error.
    """

    line_number: int
    offset: int
    message: str
    cls: type



# pylint: disable=missing-module-docstring, missing-function-docstring, line-too-long, too-few-public-methods, invalid-name, pointless-string-statement
import ast

from src.flake8_ast_error import PREFIX, Flake8ASTErrorInfo


class JsonLoadsNotAllowed:
    """
    # Bad
    import json
    response = requests.get("https://example.com")
    data = json.loads(response.text)

    # Good
    response = requests.get("https://example.com").json()
    """

    msg = PREFIX + "02: Don't use json.loads(response.text), use response.json() instead"

    @classmethod
    def check(cls, node: ast.Call, errors: list[Flake8ASTErrorInfo]) -> None:
        # pylint: disable=too-many-boolean-expressions
        if (
            isinstance(node.func, ast.Attribute)
            and isinstance(node.func.value, ast.Name)
            and node.func.value.id == "json"
            and node.func.attr == "loads"
            and isinstance(node.args[0], ast.Attribute)
            and node.args[0].attr == "text"
        ):
            errors.append(Flake8ASTErrorInfo(node.lineno, node.col_offset, cls.msg, type(cls)))



# pylint: disable=missing-module-docstring, missing-function-docstring, line-too-long, too-few-public-methods, invalid-name, pointless-string-statement
import ast

from src.flake8_ast_error import PREFIX, Flake8ASTErrorInfo


class RequestsJsonDumpsNotAllowed:
    """
    # Bad
    import json
    requests.post(data=json.dumps(data))

    # Good
    requests.post(json=data)
    """

    msg = PREFIX + "04: Don't use requests.post(data=json.dumps(data)), use requests.post(json=data) instead"

    @classmethod
    def check(cls, node: ast.Call, errors: list[Flake8ASTErrorInfo]) -> None:
        is_requests_call = (
            isinstance(node.func, ast.Attribute) and
            isinstance(node.func.value, ast.Name) and
            node.func.value.id == "requests"  # fmt: skip
        )
        if not is_requests_call:
            return
        # =================================================================================
        # From now on, we're only checking for requests.x() calls
        data = [x for x in node.keywords if x.arg == "data"]
        if not data:
            return
        data = data[0]

        if not (
            isinstance(data.value, ast.Call) and
            isinstance(data.value.func, ast.Attribute)  # fmt: skip
        ):
            return
        # ================================================================================
        # From now on, we're only checking for data=<x> calls
        attribute = data.value.func
        if (
            isinstance(attribute.value, ast.Name) and
            attribute.value.id == "json" and
            attribute.attr == "dumps"  # fmt: skip
        ):
            errors.append(Flake8ASTErrorInfo(node.lineno, node.col_offset, cls.msg, type(cls)))


"""
Call(
    func=Attribute(
        value=Name(id='requests', ctx=Load()),
        attr='post',
        ctx=Load()),
    args=[
        Constant(value='<url>')],
    keywords=[
        keyword(
            arg='data',
            value=Call(
                func=Attribute(
                    value=Name(id='json', ctx=Load()),
                    attr='dumps',
                    ctx=Load()),
                args=[
                    Name(id='<data>', ctx=Load())],
                    keywords=[]))
    ])),
"""



# pylint: disable=missing-module-docstring, missing-function-docstring, line-too-long, too-few-public-methods, invalid-name, pointless-string-statement
import ast

from src.flake8_ast_error import PREFIX, Flake8ASTErrorInfo


class AnyOrAllWithCompNotAllowed:
    """
    # Bad
    any([x.id for x in my_list])
    all([x.id for x in my_list])

    # Good
    all(x.id for x in my_list)
    any(x.id for x in my_list)
    """

    msg = PREFIX + "15: Prefer simpler any or all statements: remove unnecessary comprehension"

    @classmethod
    def check(cls, node: ast.Call, errors: list[Flake8ASTErrorInfo]) -> None:
        if (
            isinstance(node.func, ast.Name)
            and node.func.id in {"all", "any"}
            and len(node.args) == 1
            and isinstance(node.args[0], ast.ListComp)  # fmt: skip
        ):
            errors.append(Flake8ASTErrorInfo(node.lineno, node.col_offset, cls.msg, type(cls)))


"""
Call(
    func=Name(id='any', ctx=Load()),
    args=[
        ListComp(
            elt=Name(id='x', ctx=Load()),
            generators=[
                comprehension(
                    target=Name(id='x', ctx=Store()),
                    iter=Call(
                    func=Name(id='range', ctx=Load()),
                    args=[
                        Constant(value=5)],
                    keywords=[]),
                    ifs=[],
                    is_async=0)])],
        keywords=[]))],
"""


# pylint: disable=missing-module-docstring, missing-function-docstring, line-too-long, too-few-public-methods, invalid-name, pointless-string-statement
import ast

from src.flake8_ast_error import PREFIX, Flake8ASTErrorInfo


class OpenNoWithNotAllowed:
    """
    # Bad
    file = open("file.txt")
    file.read()
    file.close()

    # Good
    with open("file.txt") as file:
        data = file.read()
    """

    msg = PREFIX + "03: Don't use data = open(), use a with statement instead"

    @classmethod
    def check(cls, node: ast.Call, errors: list[Flake8ASTErrorInfo]) -> None:
        if (
            isinstance(node.func, ast.Attribute)
            and isinstance(node.func.value, ast.Call)
            and isinstance(node.func.value.func, ast.Name)
            and node.func.value.func.id == "open"
        ):
            errors.append(Flake8ASTErrorInfo(node.lineno, node.col_offset, cls.msg, type(cls)))



# pylint: disable=missing-module-docstring, missing-function-docstring, line-too-long, too-few-public-methods, invalid-name, pointless-string-statement
import ast
import re

from src.flake8_ast_error import PREFIX, Flake8ASTErrorInfo


class CamelCaseFuncNotAllowed:
    """
    # Bad
    def CamelCaseFunc():
        pass

    # Good
    def snake_case_func():
        pass
    """

    msg = PREFIX + "06: Function names should be in snake_case, not camelCase or PascalCase, (found: {}, should be: {})"

    @classmethod
    def check(cls, node: ast.For, errors: list[Flake8ASTErrorInfo]) -> None:
        if not node.name.islower():
            found = node.name
            expected = re.sub(r'(?<!^)(?=[A-Z])', '_', node.name).lower()
            errors.append(Flake8ASTErrorInfo(node.lineno, node.col_offset, cls.msg.format(found, expected), type(cls)))



# pylint: disable=missing-module-docstring, missing-function-docstring, line-too-long, too-few-public-methods, invalid-name, pointless-string-statement
import ast
import builtins

from src.flake8_ast_error import PREFIX, Flake8ASTErrorInfo

key_words = [x for x in dir(builtins) if not x.startswith("_")]


class ShadowBuiltinsNotAllowed:
    """
    # Bad
    map = ...
    list = ...

    # Good
    my_map = ...
    my_list = ...
    """

    msg = PREFIX + "13: Don't override builtins, use a different name instead (used: `{}`))"

    @classmethod
    def check(cls, node: ast.Assign, errors: list[Flake8ASTErrorInfo]) -> None:
        if (
            node.targets and
            isinstance(node.targets[0], ast.Name) and
            node.targets[0].id in key_words  # fmt: skip
        ):
            name_used = node.targets[0].id
            errors.append(Flake8ASTErrorInfo(node.lineno, node.col_offset, cls.msg.format(name_used), type(cls)))


"""
Assign(
    targets=[
        Name(id='map', ctx=Store())],
    value=Constant(value='my_map_object'))],
"""



# pylint: disable=missing-module-docstring, missing-function-docstring, line-too-long, too-few-public-methods, invalid-name, pointless-string-statement
import ast

from src.flake8_ast_error import PREFIX, Flake8ASTErrorInfo


class AssignToListNotAllowed:
    """
    # Bad
    x = list()

    # Good
    x = []
    """

    msg = PREFIX + "05: Don't use x = list(), use x = [] instead"

    @classmethod
    def check(cls, node: ast.If, errors: list[Flake8ASTErrorInfo]) -> None:
        if (
            isinstance(node.func, ast.Name) and
            node.func.id == "list" and
            not node.args and
            not node.keywords  # fmt: skip
        ):
            errors.append(Flake8ASTErrorInfo(node.lineno, node.col_offset, cls.msg, type(cls)))



# pylint: disable=missing-module-docstring, missing-function-docstring, line-too-long, too-few-public-methods, invalid-name, pointless-string-statement
import ast

from src.flake8_ast_error import PREFIX, Flake8ASTErrorInfo


class InheritsFromObjectNotAllowed:
    """
    # Bad
    class Foo(object):
        pass

    # Good
    class Foo:
        pass
    """

    msg = PREFIX + "10: Don't inherit from object, not required since Python 3.0"

    @classmethod
    def check(cls, node: ast.ClassDef, errors: list[Flake8ASTErrorInfo]) -> None:
        if any(
            base for base in node.bases
            if isinstance(base, ast.Name) and base.id == "object"  # fmt: skip
        ):
            errors.append(Flake8ASTErrorInfo(node.lineno, node.col_offset, cls.msg, type(cls)))



# pylint: disable=missing-module-docstring, missing-function-docstring, line-too-long, too-few-public-methods, invalid-name, pointless-string-statement
import ast

from src.flake8_ast_error import PREFIX, Flake8ASTErrorInfo


class CompareTypesNotAllowed:
    """
    # Bad
    if type(x) == int:
        pass
    if type(x) is int:
        pass

    # Good
    if isinstance(x, int):
        pass
    """

    msg = PREFIX + "08: Comparing types is not allowed, use isinstance() instead"

    @classmethod
    def check(cls, node: ast.Compare, errors: list[Flake8ASTErrorInfo]) -> None:
        left_is_type = (
            isinstance(node.left, ast.Call) and
            isinstance(node.left.func, ast.Name) and
            node.left.func.id == "type"  # fmt: skip
        )
        node.right = node.comparators[0]
        right_is_type = (
            isinstance(node.right, ast.Call) and
            isinstance(node.right.func, ast.Name) and
            node.right.func.id == "type"  # fmt: skip
        )
        # =========================================================
        if left_is_type or right_is_type:
            errors.append(Flake8ASTErrorInfo(node.lineno, node.col_offset, cls.msg, type(cls)))



# pylint: disable=missing-module-docstring, missing-function-docstring, line-too-long, too-few-public-methods, invalid-name, pointless-string-statement
import ast

from src.flake8_ast_error import PREFIX, Flake8ASTErrorInfo


class DefaultMutableArgsNotAllowed:
    """
    # Bad
    def foo(bar=[]):
        pass

    # Good
    def foo(bar=None):
        if bar is None:
            bar = []
        pass
    """

    msg = PREFIX + "07: Parameter defaults shouldn't be mutable, set to None instead and do if param is None: param = []"

    @classmethod
    def check(cls, node: ast.For, errors: list[Flake8ASTErrorInfo]) -> None:
        if [
            x for x in node.args.defaults
            if isinstance(x, ast.List) and
            x.elts == []  # fmt: skip
        ]:
            errors.append(Flake8ASTErrorInfo(node.lineno, node.col_offset, cls.msg, type(cls)))



# pylint: disable=missing-module-docstring, missing-function-docstring, line-too-long, too-few-public-methods, invalid-name
from _ast import Module
import ast
from typing import Any, Iterator

from src.flake8_ast_error import Flake8ASTErrorInfo

# fmt: off
from src.checks import (
    RangeLenNotAllowed, JsonLoadsNotAllowed, OpenNoWithNotAllowed, RequestsJsonDumpsNotAllowed, AssignToListNotAllowed,  # For
    CamelCaseFuncNotAllowed, DefaultMutableArgsNotAllowed, UsingFilterOrMapNotAllowed, AnyOrAllWithCompNotAllowed,  # Call
    CompareTypesNotAllowed,  # Compare
    ComparedToTrueNotAllowed, NotUsingTernaryNotAllowed,  # If
    InheritsFromObjectNotAllowed, NonPascalCaseClassNotAllowed,  # ClassDef
    ShadowBuiltinsNotAllowed,  # Assign
    NoPointlessTernaryNotAllowed,  # IfExp
    NonUsingListCompNotAllowed,  # Module
)
# fmt: on


class ProduceBetterPythonPlugin:
    """
    Plugin Class
    """

    name = "produce_better_python"
    version = "0.0.1"

    def __init__(self, tree: ast.AST) -> None:
        self._tree = tree

    def run(self) -> Iterator[Flake8ASTErrorInfo]:
        visitor = Visitor()
        visitor.visit(self._tree)
        yield from visitor.errors


class Visitor(ast.NodeVisitor):
    """
    Main visitor class
    """

    def __init__(self) -> None:
        self.errors: list[Flake8ASTErrorInfo] = []

    def visit_For(self, node: ast.For) -> None:
        RangeLenNotAllowed.check(node, self.errors)
        self.generic_visit(node)

    def visit_Call(self, node: ast.Call) -> None:
        JsonLoadsNotAllowed.check(node, self.errors)
        OpenNoWithNotAllowed.check(node, self.errors)
        RequestsJsonDumpsNotAllowed.check(node, self.errors)
        AssignToListNotAllowed.check(node, self.errors)
        UsingFilterOrMapNotAllowed.check(node, self.errors)
        AnyOrAllWithCompNotAllowed.check(node, self.errors)
        self.generic_visit(node)

    def visit_FunctionDef(self, node: ast.FunctionDef) -> Any:
        CamelCaseFuncNotAllowed.check(node, self.errors)
        DefaultMutableArgsNotAllowed.check(node, self.errors)
        self.generic_visit(node)

    def visit_ClassDef(self, node: ast.ClassDef) -> None:
        InheritsFromObjectNotAllowed.check(node, self.errors)
        NonPascalCaseClassNotAllowed.check(node, self.errors)
        self.generic_visit(node)

    def visit_Compare(self, node: ast.Compare) -> None:
        CompareTypesNotAllowed.check(node, self.errors)
        self.generic_visit(node)

    def visit_If(self, node: ast.If) -> None:
        ComparedToTrueNotAllowed.check(node, self.errors)
        NotUsingTernaryNotAllowed.check(node, self.errors)
        self.generic_visit(node)

    def visit_Assign(self, node: ast.Assign) -> None:
        ShadowBuiltinsNotAllowed.check(node, self.errors)
        self.generic_visit(node)

    def visit_IfExp(self, node: ast.IfExp) -> None:
        NoPointlessTernaryNotAllowed.check(node, self.errors)
        self.generic_visit(node)

    def visit_Module(self, node: Module) -> Any:
        NonUsingListCompNotAllowed.check(node, self.errors)
        self.generic_visit(node)


# pylint: disable=missing-module-docstring, missing-function-docstring, line-too-long, too-few-public-methods, invalid-name, pointless-string-statement
import ast

from src.flake8_ast_error import PREFIX, Flake8ASTErrorInfo, IS


class ComparedToTrueNotAllowed:
    """
    # Bad
    if x is True:
        pass

    # Good
    if x:
        pass
    """

    msg = PREFIX + "09: Don't use x is True, use bool(x) or just x: instead"

    @classmethod
    def check(cls, node: ast.If, errors: list[Flake8ASTErrorInfo]) -> None:
        if (
            isinstance(node.test, ast.Compare) and
            isinstance(node.test.ops[0], IS) and
            isinstance(node.test.comparators[0], ast.Constant) and
            node.test.comparators[0].value is True  # fmt: skip
        ):
            errors.append(Flake8ASTErrorInfo(node.lineno, node.col_offset, cls.msg, type(cls)))



# pylint: disable=missing-module-docstring, missing-function-docstring, line-too-long, too-few-public-methods, invalid-name, pointless-string-statement
import ast

from src.flake8_ast_error import PREFIX, Flake8ASTErrorInfo


class RangeLenNotAllowed:
    """
    # Bad
    for i in range(len(my_list)):
        print(my_list[i])

    # Good
    for x in list:
        print(x)
    """

    msg = PREFIX + "01: Don't use range(len(list)), use zip(list_a, list_b) instead, or for x in list"

    @classmethod
    def check(cls, node: ast.For, errors: list[Flake8ASTErrorInfo]) -> None:
        if (
            isinstance(node.iter, ast.Call) and 
            isinstance(node.iter.func, ast.Name) and
            node.iter.func.id == "range" and
            isinstance(node.iter.args[0], ast.Call) and
            isinstance(node.iter.args[0].func, ast.Name) and
            node.iter.args[0].func.id == "len"  # fmt: skip
        ):
            errors.append(Flake8ASTErrorInfo(node.lineno, node.col_offset, cls.msg, type(cls)))

"""
For(
    target=Name(id='i', ctx=Store()),
    iter=Call(
        func=Name(id='range', ctx=Load()),
        args=[
            Call(
                func=Name(id='len', ctx=Load()),
                args=[
                    Name(id='my_list', ctx=Load())],
                keywords=[])],
        keywords=[]),
    body=[
        Expr(
        value=Call(
            func=Name(id='print', ctx=Load()),
            args=[
                Subscript(
                    value=Name(id='my_list', ctx=Load()),
                    slice=Name(id='i', ctx=Load()),
                    ctx=Load()),
                Subscript(
                    value=Name(id='my_second_list', ctx=Load()),
                    slice=Name(id='i', ctx=Load()),
                    ctx=Load())],
            keywords=[]))],
    orelse=[])],
"""



# pylint: disable=missing-module-docstring
from src.checks.PBP101_range_len import RangeLenNotAllowed
from src.checks.PBP102_json_dot_loads import JsonLoadsNotAllowed
from src.checks.PBP103_open_no_with import OpenNoWithNotAllowed
from src.checks.PBP104_requests_json_dumps import RequestsJsonDumpsNotAllowed
from src.checks.PBP105_assigns_to_list import AssignToListNotAllowed
from src.checks.PBP106_camel_case_func import CamelCaseFuncNotAllowed
from src.checks.PBP107_default_mutable_args import DefaultMutableArgsNotAllowed
from src.checks.PBP108_compares_types import CompareTypesNotAllowed
from src.checks.PBP109_compares_to_true import ComparedToTrueNotAllowed
from src.checks.PBP110_inherits_from_object import InheritsFromObjectNotAllowed
from src.checks.PBP111_not_using_ternary import NotUsingTernaryNotAllowed
from src.checks.PBP112_using_filter import UsingFilterOrMapNotAllowed
from src.checks.PBP113_shadow_builtins import ShadowBuiltinsNotAllowed
from src.checks.PBP114_no_pointless_ternary import NoPointlessTernaryNotAllowed
from src.checks.PBP115_better_any_and_all import AnyOrAllWithCompNotAllowed
from src.checks.PBP116_non_pascal_case_class import NonPascalCaseClassNotAllowed
from src.checks.PBP117_use_list_comp import NonUsingListCompNotAllowed

all_checks = [
    RangeLenNotAllowed, JsonLoadsNotAllowed, OpenNoWithNotAllowed, RequestsJsonDumpsNotAllowed,
    AssignToListNotAllowed, CamelCaseFuncNotAllowed, DefaultMutableArgsNotAllowed, CompareTypesNotAllowed,
    ComparedToTrueNotAllowed, InheritsFromObjectNotAllowed, NotUsingTernaryNotAllowed, UsingFilterOrMapNotAllowed,
    ShadowBuiltinsNotAllowed, NoPointlessTernaryNotAllowed, AnyOrAllWithCompNotAllowed, NonPascalCaseClassNotAllowed,
    NonUsingListCompNotAllowed,  # fmt: skip
]


# pylint: disable=missing-module-docstring, missing-function-docstring, line-too-long, too-few-public-methods, invalid-name, pointless-string-statement
import ast

from src.flake8_ast_error import PREFIX, Flake8ASTErrorInfo, filter_by


class NotUsingTernaryNotAllowed:
    """
    # Bad
    if x == 1:
        a = "Value1"
    else:
        a = "Value2"

    # Good
    a = "Value1" if x == 1 else "Value2"
    """

    msg = PREFIX + "11: Use a ternary here, instead of an if statement"

    @classmethod
    def check(cls, node: ast.If, errors: list[Flake8ASTErrorInfo]) -> None:
        if not (
            isinstance(node.test, ast.Compare)
            and isinstance(node.test.comparators[0], ast.Constant)  # Check if we're comparing
            and node.orelse is not None,  # Against a constant  # And we have an else
        ):
            return
        # =========================================================
        # Make sure we have an else
        variable_if = filter_by(node.body, ast.Assign)
        if not variable_if:
            return
        variable_else = filter_by(node.orelse, ast.Assign)
        if not variable_else:
            return
        # =========================================================
        # We now have 2 variables, one for the if, one for the else
        if not (isinstance(variable_if.targets[0], ast.Name) and isinstance(variable_else.targets[0], ast.Name)):
            return
        if variable_if.targets[0].id != variable_else.targets[0].id:
            return

        if (
            isinstance(variable_if.value, ast.Constant) and
            isinstance(variable_else.value, ast.Constant)  # fmt: skip
        ):
            errors.append(Flake8ASTErrorInfo(node.lineno, node.col_offset, cls.msg, type(cls)))


"""
If(
    test=Compare(
        left=Name(id='x', ctx=Load()),
        ops=[
            Is()],
        comparators=[
            Constant(value=None)]),
    body=[
        Assign(
            targets=[
                Name(id='a', ctx=Store())],
            value=Constant(value=True))],
    orelse=[
        Assign(
            targets=[
                Name(id='a', ctx=Store())],
            value=Constant(value=False))
"""



# pylint: disable=missing-module-docstring, missing-function-docstring, line-too-long, too-few-public-methods, invalid-name, pointless-string-statement
import ast

from src.flake8_ast_error import PREFIX, Flake8ASTErrorInfo


class NonPascalCaseClassNotAllowed:
    """
    # Bad
    class my_class:
        pass

    # Good
    class MyClass:
        pass
    """

    msg = PREFIX + "16: Class names should be in PascalCase, (found: {}, should be: {})"

    def convert_to_pascal_case(name) -> str:
        return "".join(word.title() for word in name.split("_"))

    @classmethod
    def check(cls, node: ast.ClassDef, errors: list[Flake8ASTErrorInfo]) -> None:
        if node.name.islower() or "_" in node.name :
            found = node.name
            expected = cls.convert_to_pascal_case(node.name)
            msg = cls.msg.format(found, expected)
            errors.append(Flake8ASTErrorInfo(node.lineno, node.col_offset, msg, type(cls)))


"""
ClassDef(
    name='my_class',
    bases=[],
    keywords=[],
    body=[
    Pass()],
    decorator_list=[])],
"""


# pylint: disable=missing-module-docstring, missing-function-docstring, line-too-long, too-few-public-methods, invalid-name, pointless-string-statement
import ast

from src.flake8_ast_error import PREFIX, Flake8ASTErrorInfo


class NoPointlessTernaryNotAllowed:
    """
    # Bad
    x = True if condition else False

    # Good
    x = bool(condition)
    """

    msg = PREFIX + "14: Don't True if <condition> else False, just use bool(<condition>)"

    @classmethod
    def check(cls, node: ast.IfExp, errors: list[Flake8ASTErrorInfo]) -> None:
        if (
            isinstance(node.body, ast.NameConstant)
            and node.body.value in [True, False]
            and isinstance(node.orelse, ast.NameConstant)
            and node.orelse.value in [True, False]
        ):
            errors.append(Flake8ASTErrorInfo(node.lineno, node.col_offset, cls.msg, type(cls)))


"""
IfExp(
    test=Compare(
        left=Name(id='value', ctx=Load()),
        ops=[
            Eq()],
        comparators=[
            Constant(value=5)]),
    body=Constant(value=True),
    orelse=Constant(value=False)))],
"""



# pylint: disable=missing-module-docstring, missing-function-docstring, line-too-long, too-few-public-methods, invalid-name, pointless-string-statement
import ast

from src.flake8_ast_error import PREFIX, Flake8ASTErrorInfo


class NonUsingListCompNotAllowed:
    """
    # Bad
    my_list = []
    for char in my_sub_list:
        my_list.append(char)

    # Good
    my_list = [x for x in my_sub_list]
    """

    msg = PREFIX + "17: Use list comprehension where appropriate"

    @classmethod
    def check(cls, node: ast.Module, errors: list[Flake8ASTErrorInfo]) -> None:
        for i, statement in enumerate(node.body):
            if i == len(node.body) - 1:
                break
            next_statement = node.body[i + 1]
            # Generate the required AST conditions to disallow looping and appending where list comp would work instead
            if (
                # Check if they're assigning to a list
                isinstance(statement, ast.Assign) and
                isinstance(statement.value, ast.List) and
                statement.targets and
                isinstance(statement.targets[0], ast.Name) and
                # =========================================================
                # Check if the next statement is a for loop
                isinstance(next_statement, ast.For) and
                isinstance(next_statement.body[0], ast.Expr) and
                isinstance(next_statement.body[0].value, ast.Call) and
                isinstance(next_statement.body[0].value.func, ast.Attribute) and
                isinstance(next_statement.body[0].value.func.value, ast.Name) and
                next_statement.body[0].value.func.value.attr == "append" and
                next_statement.body[0].value.func.value.id == statement.targets[0].id
            ):
                errors.append(Flake8ASTErrorInfo(node.lineno, node.col_offset, cls.msg, type(cls)))


"""
Assign(
    targets=[
        Name(id='my_list', ctx=Store())],
    value=List(elts=[], ctx=Load())),
For(
    target=Name(id='char', ctx=Store()),
    iter=Constant(value='hello'),
    body=[
        Expr(
            value=Call(
                func=Attribute(
                    value=Name(id='my_list', ctx=Load()),
                    attr='append',
                    ctx=Load()),
                args=[
                    Name(id='char', ctx=Load())],
                keywords=[]))],
    orelse=[])],
"""


from setuptools import setup, find_packages

setup(
    install_requires=["flake8 > 3.0.0"],
    name="flake8_pbp",
    license="MIT",
    version="1.0.5",
    description="flake8 plugin which checks for various bad python practises",
    long_description="flake8 plugin which checks for various bad python practises",
    url="https://github.com/UP929312/pbp",
    packages=find_packages(),
    classifiers=[
        "Framework :: Flake8",
    ],
    entry_points={
        "flake8.extension": [
            "PBP1 = src.flake8_pbp:ProduceBetterPythonPlugin",
        ],
    },
)



import requests

condition = True
my_list = [1, 2, 3]
my_map = {1: 'a', 2: 'b'}

#========== RangeLenNotAllowed

# Bad
for i in range(len(my_list)):
    print(my_list[i])

# Good
for x in list:
    print(x)

#========== JsonLoadsNotAllowed

# Bad
import json
response = requests.get("https://example.com")
data = json.loads(response.text)

# Good
response = requests.get("https://example.com").json()

#========== OpenNoWithNotAllowed

# Bad
file = open("file.txt")
file.read()
file.close()

# Good
with open("file.txt") as file:
    data = file.read()

#========== RequestsJsonDumpsNotAllowed

# Bad
import json
requests.post(data=json.dumps(data))

# Good
requests.post(json=data)

#========== AssignToListNotAllowed

# Bad
x = list()

# Good
x = []

#========== CamelCaseFuncNotAllowed

# Bad
def CamelCaseFunc():
    pass

# Good
def snake_case_func():
    pass

#========== DefaultMutableArgsNotAllowed

# Bad
def foo(bar=[]):
    pass

# Good
def foo(bar=None):
    if bar is None:
        bar = []
    pass

#========== CompareTypesNotAllowed

# Bad
if type(x) == int:
    pass
if type(x) is int:
    pass

# Good
if isinstance(x, int):
    pass

#========== ComparedToTrueNotAllowed

# Bad
if x is True:
    pass

# Good
if x:
    pass

#========== InheritsFromObjectNotAllowed

# Bad
class Foo(object):
    pass

# Good
class Foo:
    pass

#========== NotUsingTernaryNotAllowed

# Bad
if x == 1:
    a = "Value1"
else:
    a = "Value2"

# Good
a = "Value1" if x == 1 else "Value2"

#========== UsingFilterOrMapNotAllowed

# Bad
list(filter(lambda x: x > 5, range(10)))
list(map(lambda x: x**2, range(10)))

# Good
[x for x in range(10) if x > 5]
[x**2 for x in range(10)]

#========== ShadowBuiltinsNotAllowed

# Bad
map = ...
list = ...

# Good
my_map = ...
my_list = ...

#========== NoPointlessTernaryNotAllowed

# Bad
x = True if condition else False

# Good
x = bool(condition)

#========== AnyOrAllWithCompNotAllowed

# Bad
any([x.id for x in my_list])
all([x.id for x in my_list])

# Good
all(x.id for x in my_list)
any(x.id for x in my_list)

#========== NonPascalCaseClassNotAllowed

# Bad
class my_class:
    pass

# Good
class MyClass:
    pass

